#include "SceneGraph.h"
#include "Exporter.h"
#include "UnigraphicsParser.h"
#include "X3dParser.h"
#include "Preprocessor.h"
#include "ViewCell.h"
#include "Environment.h"

Preprocessor::Preprocessor():
mKdTree(NULL),
mBspTree(NULL),
mRootViewCell(NULL)
{
}

Preprocessor::~Preprocessor()
{
  CLEAR_CONTAINER(mViewCells);
  DEL_PTR(mBspTree);
  DEL_PTR(mKdTree);
  DEL_PTR(mRootViewCell);
}

bool
Preprocessor::LoadViewCells(const string filename)
{
	return X3dParser().ParseFile(filename, mViewCells);
}

bool
Preprocessor::GenerateViewCells()
{
	// TODO
	// HACK: derive view cells from the scene objects
	ObjectContainer objects;

	int maxViewCells = 0;
	environment->GetIntValue("BspTree.maxViewCells", maxViewCells);

	mSceneGraph->CollectObjects(&objects);
	ViewCell::DeriveViewCells(objects, mViewCells, maxViewCells);

	return true;
}

bool
Preprocessor::LoadScene(const string filename)
{
  // use leaf nodes of the original spatial hiearrchy as occludees

  mSceneGraph = new SceneGraph;

  
  Parser *parser;

  if (strstr(filename.c_str(), ".x3d"))
    parser = new X3dParser;
  else
    parser = new UnigraphicsParser;

  bool result = parser->ParseFile(filename, &mSceneGraph->mRoot);
  
  delete parser;

  return result;
}

bool
Preprocessor::ExportPreprocessedData(const string filename)
{
  return false;
}

bool
Preprocessor::BuildKdTree()
{
  mKdTree = new KdTree;
  // add mesh instances of the scene graph to the root of the tree
  KdLeaf *root = (KdLeaf *)mKdTree->GetRoot();
  mSceneGraph->CollectObjects(&root->mObjects);
  
  mKdTree->Construct();
  return true;
}


bool
Preprocessor::BuildBspTree()
{
	DEL_PTR(mBspTree);
	DEL_PTR(mRootViewCell);
	mRootViewCell = new ViewCell();
	mBspTree = new BspTree(mRootViewCell);

	ObjectContainer objects;
	RayContainer rays;

	switch (BspTree::sConstructionMethod)
	{
	case BspTree::VIEW_CELLS:
		mBspTree->Construct(mViewCells);
		break;
	case BspTree::SCENE_GEOMETRY:
		CLEAR_CONTAINER(mViewCells); // we generate new view cells

		mSceneGraph->CollectObjects(&objects);
		mBspTree->Construct(objects, &mViewCells);
		break;
	case BspTree::RAYS:
		CLEAR_CONTAINER(mViewCells); // we generate new view cells	
		mBspTree->Construct(rays, &mViewCells);
		break;
	default:
		Debug << "Error: Method not available\n";
		break;
	}
	return true;
}


void
Preprocessor::KdTreeStatistics(ostream &s)
{
  s<<mKdTree->GetStatistics();
}

void
Preprocessor::BspTreeStatistics(ostream &s)
{
	s << mBspTree->GetStatistics();
}

bool
Preprocessor::Export( const string filename,
		      const bool scene,
		      const bool kdtree,
			  const bool bsptree
		      )
{
  Exporter *exporter = Exporter::GetExporter(filename);

  if (exporter) {
    if (scene)
      exporter->ExportScene(mSceneGraph->mRoot);

    if (kdtree) {
      exporter->SetWireframe();
      exporter->ExportKdTree(*mKdTree);
    }

	if (bsptree) {
		//exporter->SetWireframe();
		exporter->ExportBspTree(*mBspTree);
	}

    delete exporter;
    return true;
  }

  return false;
}



